const express = require("express");
const router = express.Router();
require("dotenv").config();
const StellarSDK = require("stellar-sdk");
const DecryptWithServerKey = require("./../../services/decryptionservice");
const EncryptionService = require("./../../services/encryptionservice");
const AccountCreateCollection = require("../../model/stellar/accountCreate");
const UserSchema = require("../../model/patient/patient");
const { generateKeyPairSync } = require("crypto");
const GenerateRandomAssetCode = require("./../../services/assetCodeGenerator");
const GenerateKeyPairAndCreateAccount = require("./../../services/accountCreator");
let timestamp;

router.post("/", async (req, res) => {
  console.log(req.body);
  //decrypt user request details
  const decryptedBirthday = DecryptWithServerKey(req.body.birthday);
  const decryptedChildCount = DecryptWithServerKey(req.body.childcount);
  const decryptedEducationLevel = DecryptWithServerKey(req.body.educationlevel);
  const decryptedEmail = DecryptWithServerKey(req.body.email);
  const decryptedEmployed = DecryptWithServerKey(req.body.employed);
  const decryptedEthnicity = DecryptWithServerKey(req.body.ethnicity);
  const decryptedGender = DecryptWithServerKey(req.body.gender);
  const decryptedName = DecryptWithServerKey(req.body.name);
  const decryptedNationality = DecryptWithServerKey(req.body.nationality);
  const decryptedOrphan = DecryptWithServerKey(req.body.orphan);
  const decryptedPassword = req.body.password;
  const decryptedPronoun = DecryptWithServerKey(req.body.pronoun);
  const decryptedRelationship = DecryptWithServerKey(req.body.relationship);
  const decryptedReligion = DecryptWithServerKey(req.body.religion);
  const decryptedSexualPreference = DecryptWithServerKey(
    req.body.sexualpreference
  );
  const decryptedSiblingCount = DecryptWithServerKey(req.body.siblingcount);
  const decryptedUserName = DecryptWithServerKey(req.body.username);

  //get the server key pair
  const serverSourceKeyPair = StellarSDK.Keypair.fromSecret(
    process.env.STELLARSEED
  );
  const serverPubKey = serverSourceKeyPair.publicKey();
  const server = new StellarSDK.Server(process.env.STELLARTESTNET);

  //load server account
  const account = await server.loadAccount(serverPubKey);
  const fee = await server.fetchBaseFee();

  //generate random keypair
  const userKeyPair = StellarSDK.Keypair.random();

  //build the account funding transaction
  const accCreateTransaction = new StellarSDK.TransactionBuilder(account, {
    fee,
    networkPassphrase: StellarSDK.Networks.TESTNET,
  })
    .addMemo(StellarSDK.Memo.text("Create account"))
    .addOperation(
      StellarSDK.Operation.createAccount({
        destination: userKeyPair.publicKey(),
        startingBalance: "100",
      })
    )
    .setTimeout(30)
    .build();

  //sign the transaction
  accCreateTransaction.sign(serverSourceKeyPair);

  //send account creation transaction to BC
  try {
    const accCreateTxnResult = await server.submitTransaction(
      accCreateTransaction
    );
    console.log("New user account created: HASH : ", accCreateTxnResult.hash);
    console.log(
      "View account creation transaction at : https://horizon-testnet.stellar.org/transactions/" +
        accCreateTxnResult.hash
    );
    timestamp = new Date();
    //create the account creation details collection object
    const createAccountObj = new AccountCreateCollection({
      publickey: userKeyPair.publicKey(),
      amount: "10XLM",
      timestamp: timestamp,
      transactionhash: accCreateTxnResult.hash,
    });

    //save the account creation details to the database
    try {
      const saveInCollection = await createAccountObj.save();
      console.log(
        "Account creation details saved in the database " + saveInCollection
      );

      //get the user keypair
      const userSourceKeyPair = StellarSDK.Keypair.fromSecret(
        userKeyPair.secret()
      );
      const userSourcePublicKey = userSourceKeyPair.publicKey();

      //load user account
      const userAccount = await server.loadAccount(userSourcePublicKey);

      //build genesis transaction
      const genesisTransaction = new StellarSDK.TransactionBuilder(
        userAccount,
        {
          fee,
          networkPassphrase: StellarSDK.Networks.TESTNET,
        }
      )
        .addMemo(StellarSDK.Memo.text("Genesis"))
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Email Address",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedEmail,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Name",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedName,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Pronouns",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedPronoun,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Birthday",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedBirthday,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Gender",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedGender,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Orphan",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedOrphan,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Sibling Count",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedSiblingCount,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Relationship Status",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedRelationship,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Child Count",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedChildCount,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Sexual Preference",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedSexualPreference,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Religion",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedReligion,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Ethnicity",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedEthnicity,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Nationality",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedNationality,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Education level",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedEducationLevel,
              userKeyPair.secret()
            ),
          })
        )
        .addOperation(
          StellarSDK.Operation.manageData({
            name: "Employed",
            value: EncryptionService.EncryptWithStellarSeed(
              decryptedEmployed,
              userKeyPair.secret()
            ),
          })
        )
        .setTimeout(30)
        .build();

      //sign the genesis transaction
      genesisTransaction.sign(userSourceKeyPair);

      //send the genesis transaction
      try {
        const genesisTxnResult = await server.submitTransaction(
          genesisTransaction
        );
        console.log(
          "Genesis transaction submitted: HASH : ",
          genesisTxnResult.hash
        );
        console.log(
          "View genesis transaction at : https://horizon-testnet.stellar.org/transactions/" +
            genesisTxnResult.hash
        );
        timestamp = new Date();

        //generate RSA key pair
        const keyOptions = [
          {
            modulusLength: 2048,
            publicKeyEncoding: {
              type: "spki",
              format: "pem",
            },
            privateKeyEncoding: {
              type: "pkcs8",
              format: "pem",
              cipher: "aes-256-cbc",
              passphrase: "top secret",
            },
          },
        ];

        const [{ publicKey: RSAPK, privateKey: RSASK }] = keyOptions.map(
          (options) => generateKeyPairSync("rsa", options)
        );

        //base64 encoding of the key
        const b64RSApk = btoa(RSAPK);
        const b64RSAsk = btoa(RSASK);

        //generate new account
        try {
          const result = await GenerateKeyPairAndCreateAccount();
          console.log("Issuer Public Key:", result.publicKey);
          console.log("Issuer Private Key:", result.privateKey);

          //create the user collection object to be added to the DB
          const userCollectionObject = new UserSchema({
            username: decryptedUserName,
            password: decryptedPassword,
            email: decryptedEmail,
            publickey: userKeyPair.publicKey(),
            rsa: b64RSApk,
            genesishash: genesisTxnResult.hash,
            currenthash: genesisTxnResult.hash,
            timestamp: timestamp,
            issuerpk: EncryptionService.EncryptWithServerKey(result.publicKey),
            issuersk: EncryptionService.EncryptWithServerKey(result.privateKey),
          });

          //save the user collection object to the database
          try {
            const saveUserInCollection = await userCollectionObject.save();
            console.log("User details added to the DB " + saveUserInCollection);
            console.log("Public key: " + userKeyPair.publicKey());
            console.log("Secret: " + userKeyPair.secret());
            res.status(200).json({
              message: "User details added to the blockchain",
              hash: genesisTxnResult.hash,
              pubkey: EncryptionService.EncryptWithServerKey(
                userKeyPair.publicKey()
              ),
              seed: EncryptionService.EncryptWithServerKey(
                userKeyPair.secret()
              ),
              rsapk: EncryptionService.EncryptWithServerKey(b64RSApk),
              rsask: EncryptionService.EncryptWithServerKey(b64RSAsk),
            });
            console.log(
              "/patient/register - register patient details successful"
            );
          } catch (errorWhenAddingUserToDB) {
            console.log(
              "Error when adding user details to the database : " +
                errorWhenAddingUserToDB
            );
            res.status(500).json({
              message:
                "Error when adding user details to the database : " +
                errorWhenAddingUserToDB,
            });
            console.log(
              "/patient/register - register patient details unsuccessful"
            );
          }
        } catch (errWhenGeneratingIssuerAccount) {
          console.log(
            "Error when generating issuer account : " +
              errWhenGeneratingIssuerAccount
          );
          res.status(500).json({
            message:
              "Error when generating issuer account : " +
              errWhenGeneratingIssuerAccount,
          });
          console.log(
            "/patient/register - register patient details unsuccessful"
          );
        }
      } catch (error) {
        console.log(
          "Error when sending genesis transaction to blockchain: " + error
        );
        res.status(500).json({
          message:
            "Error when sending genesis transaction to blockchain: " + error,
        });
        console.log(
          "/patient/register - register patient details unsuccessful"
        );
      }
    } catch (error) {
      console.log(
        "Error when sending account creation transaction details to DB : " +
          error
      );
      res.status(500).json({
        message:
          "Error when sending account creation transaction details to DB : " +
          error,
      });
      console.log("/patient/register - register patient details unsuccessful");
    }
  } catch (error) {
    console.log(
      "Error when sending account creation transaction to blockchain: " + error
    );
    res.status(500).json({
      message:
        "Error when sending account creation transaction to blockchain: " +
        error,
    });
    console.log("/patient/register - register patient details unsuccessful");
  }
});

module.exports = router;
