import React, { useState } from "react";
import { View, TextInput, Button, Alert } from "react-native";
import { LOCALBACKEND } from "../../env";
import EncryptWithServerKey from "../../services/encryptByServerKey";

export default function PractitionerLogin({ navigation, route }) {
  const [username, setUsername] = useState("");
  const [password, setPassword] = useState("");

  const handleLogin = async () => {
    const encryptedUsername = EncryptWithServerKey(username);
    const encryptedPassword = EncryptWithServerKey(password);
    try {
      const loginUrl = LOCALBACKEND + "/practitioner/login";
      const response = await fetch(loginUrl, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({
          username: encryptedUsername,
          password: encryptedPassword,
        }),
      });

      if (response.ok) {
        navigation.navigate("PractitionerMainView");
      } else {
        // Login failed, show an error message
        Alert.alert("Login Failed", "Invalid username or password");
      }
    } catch (error) {
      console.error(error);
      // Handle any network or server errors
      Alert.alert("Error", "An error occurred while logging in");
    }
  };

  return (
    <View>
      <TextInput
        placeholder="Username"
        value={username}
        onChangeText={setUsername}
      />
      <TextInput
        placeholder="Password"
        value={password}
        onChangeText={setPassword}
        secureTextEntry
      />
      <Button title="Login" onPress={handleLogin} />
    </View>
  );
}
